Fraese = {};

function Fraese.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Fillable, specializations) and SpecializationUtil.hasSpecialization(TurnOnVehicle, specializations)  and SpecializationUtil.hasSpecialization(Shovel, specializations);
end;

function Fraese:load(savegame)
    self.setUnitFillLevel = Utils.overwrittenFunction(self.setUnitFillLevel, Fraese.setUnitFillLevel);

    self.fraese = {};

    self.fraese.effects = EffectManager:loadEffect(self.xmlFile, "vehicle.fraeseEffect", self.components, self);
    self.fraese.currentDelay = 0;
    for _, effect in pairs(self.fraese.effects) do
        if effect.planeFadeTime ~= nil then
            self.fraese.currentDelay = self.fraese.currentDelay + effect.planeFadeTime;
        end
    end
    self.fraese.maxDelay = self.fraese.currentDelay

    self.fraese.turnedOnRotationNodes = Utils.loadRotationNodes(self.xmlFile, {}, "vehicle.turnedOnRotationNodes.turnedOnRotationNode", "fraese", self.components);
   
    self.fraese.startFillLevel = self:getUnitCapacity(self.overloading.fillUnitIndex) * Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.fraese#startPercentage"), 0.9);

    local i = 0
    while true do
        local key = string.format("vehicle.fraese.offset(%d)", i);
        if not hasXMLProperty(self.xmlFile, key) then
            break
        end

        local movingToolNode = Utils.indexToObject(self.components, getXMLString(self.xmlFile, key.."#movingToolNode"))
        if movingToolNode ~= nil and self.nodesToMovingTools[movingToolNode] ~= nil then
            if self.fraese.offsets == nil then
                self.fraese.offsets = {}
            end

            local offset = {}
            offset.movingTool = self.nodesToMovingTools[movingToolNode]
            offset.lastState = 0;
            offset.effects = {}
            local j = 0
            while true do
                local effectKey = string.format(key..".effect(%d)", j);
                if not hasXMLProperty(self.xmlFile, effectKey) then
                    break
                end

                local effectIndex = Utils.getNoNil(getXMLInt(self.xmlFile, effectKey.."#index"), 0)
                if self.fraese.effects[effectIndex] ~= nil and self.fraese.effects[effectIndex].setOffset ~= nil then
                    local effect = {}
                    effect.effect = self.fraese.effects[effectIndex]
                    effect.minValue = Utils.getNoNil(getXMLFloat(self.xmlFile, effectKey.."#minOffset"), 0)*1000
                    effect.maxValue = Utils.getNoNil(getXMLFloat(self.xmlFile, effectKey.."#maxOffset"), 1)*1000
                    effect.inverted = Utils.getNoNil(getXMLBool(self.xmlFile, effectKey.."#inverted"), false)
                    table.insert(offset.effects, effect)
                end
                j = j + 1
            end

            table.insert(self.fraese.offsets, offset)
        end
        i = i + 1
    end

    self.fraese.lastLoadingTime = 0;
    self.fraese.lastUnloadingTime = 0;
    self.fraese.lastFillLevelChangedTime = 0;
    self.fraese.lastFilllevel = 0;

    if table.getn(self.fraese.effects) > 0 then
        self.getAllowDelayedUnloading = Fraese.getAllowDelayedUnloading;
        self.getAllowDelayEffects = Fraese.getAllowDelayEffects;
    end;

    self.fillLevelChangedDirtyFlag = self:getNextDirtyFlag();
end;

function Fraese:delete()
	    if self.isClient then
	        EffectManager:deleteEffects(self.fraese.effects);
	    end
	end

	function Fraese:readUpdateStream(streamId, timestamp, connection)
	    if connection:getIsServer() then
	        self.fraese.lastFillLevelChangedTime = streamReadInt32(streamId);
	    end
	end

	function Fraese:writeUpdateStream(streamId, connection, dirtyMask)
	    if not connection:getIsServer() then
	        streamWriteInt32(streamId, self.fraese.lastFillLevelChangedTime)
	    end
	end

	function Fraese:update(dt)
	    if self.isClient then
	        Utils.updateRotationNodes(self, self.fraese.turnedOnRotationNodes, dt, self.fraese.lastFillLevelChangedTime + 100 > g_currentMission.time);
	    end;
	end

	function Fraese:updateTick(dt)
	    
	end

	function Fraese:setUnitFillLevel(superFunc, fillUnitIndex, fillLevel, fillType, force, fillInfo)
	    local isLoading = fillLevel > self.fraese.lastFilllevel;
	    local isUnLoading = fillLevel < self.fraese.lastFilllevel;
	
	    if fillUnitIndex == self.overloading.fillUnitIndex then
	        if self.isClient then
	            if self.fraese.effects ~= nil then
	                if fillType ~= FillUtil.FILLTYPE_UNKNOWN then
	                    EffectManager:setFillType(self.fraese.effects, fillType)
	                    EffectManager:startEffects(self.fraese.effects)
	                else
	                    EffectManager:stopEffects(self.fraese.effects)
	                end;
	            end
	
	            for _, effect in pairs(self.fraese.effects) do
	                if effect.setMorphPosition ~= nil then
	                    if isLoading then
	                        local globalPos = Utils.clamp(fillLevel/self.fraese.startFillLevel, 0, 1)
	                        local localPos = (globalPos - (effect.startDelay/self.fraese.currentDelay)) / (effect.planeFadeTime/self.fraese.currentDelay);
	                        local offset = effect.offset/effect.planeFadeTime;
	
	                        effect:setMorphPosition(offset, Utils.clamp(localPos+offset, offset, 1));
	                    elseif isUnLoading and not isLoading then
	                        local globalPos = 1-Utils.clamp(fillLevel/self.fraese.startFillLevel, 0, 1);
	                        local localPos = (globalPos - (effect.startDelay/self.fraese.currentDelay)) / (effect.planeFadeTime/self.fraese.currentDelay);
	                        local offset = effect.offset/effect.planeFadeTime;
	
	                        effect:setMorphPosition(Utils.clamp(localPos+offset, offset, 1), 1)
	                    end;
	                end
	            end
	
	           
	
	            if self.fraese.offsets ~= nil then
	                local updateDelay = false;
	                for _, offset in pairs(self.fraese.offsets) do
	                    local movingTool = offset.movingTool
	                    local state = Cylindered.getMovingToolState(self, movingTool);
	                    if state ~= offset.lastState then
	                        for _, effect in pairs(offset.effects) do
	                            local effectState = state
	                            if effect.inverted then
	                                effectState = 1 - effectState
	                            end
	
	                            effect.effect:setOffset(Utils.lerp(effect.minValue, effect.maxValue, effectState));
	                            updateDelay = true;
	                        end
	
	                        offset.lastState = state;
	                    end;
	                end
	
	                if updateDelay then
	                    self.fraese.currentDelay = 0;
	                    for _, effect in pairs(self.fraese.effects) do
	                        if effect.planeFadeTime ~= nil then
	                            self.fraese.currentDelay = self.fraese.currentDelay + effect.planeFadeTime - effect.offset;
	                        end
	                    end
	                end;
	            end
	        end;
	    end;
	
	    if isLoading then
	        self.fraese.lastLoadingTime = g_currentMission.time;
	    elseif isUnLoading then
	        self.fraese.lastUnloadingTime = g_currentMission.time;
	    end;
	
	    self.fraese.lastFilllevel = fillLevel;
	
	    if self.isServer then
	        self.fraese.lastFillLevelChangedTime = g_currentMission.time;
	        self:raiseDirtyFlags(self.fillLevelChangedDirtyFlag);
	    end;
	
	    if superFunc ~= nil then
	        return superFunc(self, fillUnitIndex, fillLevel, fillType, force, fillInfo);
	    end
	    return nil;
	end

	function Fraese:getAllowDelayedUnloading()
	    local fillLevel = self:getUnitFillLevel(self.overloading.fillUnitIndex);
	
	    if fillLevel > self.fraese.startFillLevel then
	        return true;
	    end;
	
	    if fillLevel > 0 and self.fraese.lastLoadingTime + 250 < g_currentMission.time and self:getOverloadingActive() then
	        return true;
	    end;
	
	    return false;
	end

	function Fraese:getAllowDelayEffects()
	    return self.lastFillLevelChangeTime + 250 > g_currentMission.time and self.fraese.lastUnloadingTime + 250 > g_currentMission.time;
	end
	
	function Fraese:getUnitCapacity(superFunc)
	return 0;
	end;
	
	function Fraese:draw()
    end;
	function Fraese:mouseEvent(posX, posY, isDown, isUp, button)
	end;
	function Fraese:keyEvent(unicode, sym, modifier, isDown)
	end;